from player import Player, AVAILABLE_QUALITY_LEVELS, VIDEO_CHUNCK_LEN
import matplotlib.pyplot as plt
import pandas as pd
import numpy as np

def plot(results, figsize=(20, 7)):
    plt.figure(figsize=figsize)
    plt.subplot("311")

    for r in results:
        plt.plot(r.times, r.buffer_levels, label=r.name)
    plt.ylabel("Buffer Level (s)")
    plt.legend(loc="upper right")
    plt.grid()

    plt.subplot("312")
    for r in results:
        plt.step(r.times, r.bitrates, label=r.name)
    plt.xlabel("Time (s)")
    plt.ylabel("Downloaded Bitrate (Mbps)")
    plt.legend(loc="upper right")
    plt.grid()

    plt.subplot("313")
    for r in results:
        plt.step(r.times, r.qualities, label=r.name)
    plt.xlabel("Time (s)")
    plt.ylabel("Current Quality Level")
    plt.legend(loc="upper right")
    plt.grid()

    
    plt.show()

def evaluate(abrs, tc_team_abr_parameters, BUFFER_SIZE_MS, BUFFER_PLAYBACK_START_THRESHOLD_MS, RANDOM_SEED):
    results = []
    for abr in abrs:
        abr_algorithm = abr(VIDEO_CHUNCK_LEN, AVAILABLE_QUALITY_LEVELS, **tc_team_abr_parameters)
        results.append(Player(abr_algorithm, BUFFER_SIZE_MS, BUFFER_PLAYBACK_START_THRESHOLD_MS, RANDOM_SEED).playback())

    plot(results)

    table = map(lambda res: res.to_row(), results)
    headers = ["ABR", "Avg. buffer level (s)", "Avg. bitrate (Mbps)", "Total rebuffering (s)", "Total time wasted (s)", "Positive switches", "Negative switches", "QoE"]

    df = pd.DataFrame(table, columns=headers).set_index(["ABR"])

    df["Result"] = df["QoE"] == df["QoE"].max()
    df["Result"] = list(map(lambda x: "&#128526;" if x else "&#128545;", df["Result"]))
    def highlight_max(s):
        marked_cells = []
        if s.name in ["Avg. bitrate (Mbps)", "QoE"]:
            marked_cells = s == s.max()
        else:
            marked_cells = s == s.min()
        return ['color: green; font-weight: bold' if v else '' for v in marked_cells]

    df = df.style.apply(highlight_max)
    return df
